
#include "/lib/fog/fog_common.glsl"
vec3 shadowSpace0()
{
    vec3 p3 = gbufferModelViewInverse[3].xyz;
    p3 = mat3(shadowModelView) * p3 + shadowModelView[3].xyz;
    p3 = diagonal3(shadowProjection) * p3 + shadowProjection[3].xyz;
    return p3;
}

vec3 getVolumetricRays3(vec3 solid, vec4 translucent, inout vec3 sky, float border, vec3 fragpos, vec3 fragpos0,
                        bool iswater, float light)
{

    float estEyeDepth = clamp((14.0 - eyeBrightnessSmooth.y * (16.0 / 255.0)) * (1.0 / 14.0), 0.0, 1.0);
    estEyeDepth = estEyeDepth * estEyeDepth * estEyeDepth * 34.0;

    vec3 absorbance_w = vec3(1.0);
    vec3 vL_a = vec3(0.0);
    vec3 vL_w = vec3(0.0);
    vec3 transmittance;
    float rayLength;

    vec3 refractedSunVec = refract(lightPos, -vec3(0.0, 1.0, 0.0), 0.9);
    vec3 fragposmixed = mix(fragpos0, fragpos, 1.0);
    float SdotV = dot(sunVec, normalize(fragpos));

#ifdef OVERWORLD

    if (isEyeInWater == 1)
    {
        float maxZ = min(length(fragpos), 32.0) / (1e-8 + length(fragpos));

        rayLength = length(fragpos) * maxZ;
    }
    else
    {
        rayLength = gbf(fragposmixed, far, 0) * 1028;
        if (iswater)
        {
            rayLength = mix(max(rayLength, 16), max(rayLength, 2), rainStrength);
        }
    }
#endif

    if (!(isEyeInWater == 0 && !iswater))
        AtmosphereNoLoop(6, 11.0, rayLength, normalize(mat3(gbufferModelViewInverse) * fragposmixed).y * rayLength,
                         phaseg(SdotV, 0.4), phaseRayleigh(SdotV), estEyeDepth, refractedSunVec, atmosphereUp, sunLight,
                         absorbance_w, vL_w);

    vL_a = vL_a + (vL_a - dot(vL_a, vec3(0.299, 0.587, 0.114))) * 0.1;

#ifdef OVERWORLD

    float rayLengthGround = pow(length(fragposmixed), 1.75) * fogAmount1 * mix(1, 10, rainStrength);
#else
    float rayLengthGround = pow(length(fragposmixed), 2.5);

#endif

    vL_a =
        fogAtmosphere2(mat3(gbufferModelViewInverse) * fragposmixed, rayLengthGround, rayLengthGround, transmittance);

    switch (isEyeInWater)
    {
    case 0:
        if (iswater)
        {

            vec3 transformedFragPos = mat3(gbufferModelViewInverse) * fragpos;
            float transformedFragPosLength = length(transformedFragPos);
            float np3_y = transformedFragPos.y / transformedFragPosLength;

            float estimatedDepth = distance(fragpos, fragpos0) * abs(np3_y);

            absorbance_w = min(exp(-(totEpsilon * 3.14) * estimatedDepth), 1);

            solid *= absorbance_w;
            solid += vL_w * light;
        }
        translucent.rgb *= transmittance;
        translucent.rgb += vL_a;

        solid *= transmittance;
        solid += vL_a;

        break;
    case 1:
        absorbance_w = exp(-length(fragpos) * totEpsilon);

        solid *= absorbance_w;
        solid += vL_w;
        translucent.rgb *= absorbance_w;
        translucent.rgb += vL_w;
        sky *= absorbance_w;
        sky += vL_w;
        break;
    case 2:
    case 3:
        solid *= exp(-length(fragpos) * 1);
        solid += fogColor;
        break;
    }
#if !defined DISTANT_HORIZONS && defined SKYBLEND

    float blend_solid = gbf(fragpos, far, 0.0);
    float blend_transparent = gbf(fragpos, far, 0.0);
    solid = mix(solid, sky, blend_solid * blend_solid);
    translucent.rgb = mix(translucent.rgb, sky, blend_transparent * blend_transparent);
#endif

    vec3 color = mix(solid, translucent.rgb, translucent.a);

    return vec3(color);
}

float GetBorderFogMixFactor(in vec3 eyePlayerPos, in float far, in float z0)
{

    float eyeDist = length(eyePlayerPos);
    float borderFogFactor = smoothstep(far * 0.5, far, eyeDist);
    borderFogFactor *= borderFogFactor * borderFogFactor;
    return (eyeDist > far * 0.5) ? borderFogFactor : 0.0;
}