
uniform sampler2DShadow shadow;

uniform ivec2 eyeBrightnessSmooth;

uniform float darknessInv;
uniform float darknessEye;
uniform float blindness;
uniform float darknessFactor;
uniform float normWT;
uniform mat4 shadowModelView;
uniform mat4 shadowProjection;

#include "/lib/shadow_param.glsl"

float phaseRayleigh(float cosTheta)
{
    return cosTheta * 0.18 + 0.51;
}

float phaseg(float x, float g)
{
    float gg = g * g;
    return (gg * -0.25 + 0.25) * pow(-2.0 * (g * x) + (gg + 1.0), -1.5) / PI;
}

vec3 calculateAbsorbance(vec3 scatterCoeff, float attenuationDepth)
{
    return clamp(exp(-scatterCoeff * attenuationDepth), 0.0, 1.0);
}
float phaseRayleigh0(float cosTheta)
{
    const vec2 mul_add = vec2(0.1, 0.28) / acos(-1.0);
    return cosTheta * mul_add.x + mul_add.y; // optimized version from [Elek09], divided by 4 pi for energy conservation
}
float phaseRayleigh1(float cosTheta)
{
    const vec2 mul_add = vec2(0.1, 0.28) / acos(-1.0);
    return cosTheta * mul_add.x + mul_add.y; // optimized version from [Elek09], divided by 4 pi for energy conservation
}

#define diagonal3(m) vec3((m)[0].x, (m)[1].y, m[2].z)

vec3 toShadowSpaceProjected(vec3 p3)
{
    p3 = mat3(gbufferModelViewInverse) * p3 + gbufferModelViewInverse[3].xyz;
    p3 = mat3(shadowModelView) * p3 + shadowModelView[3].xyz;
    p3 = diagonal3(shadowProjection) * p3 + shadowProjection[3].xyz;

    return p3;
}

#include "/lib/sky_function.glsl"

const vec3 totEpsilon = vec3(0.69, 0.19, 0.14);

const vec3 dirtScatterCoef = 0.1 * vec3(0.2, 0.6, 0.8);

float safety = 0.00000001;

vec3 water_coefficientRayleigh = vec3(1.0 * 1e-6, 2.5 * 1e-6, 3.5 * 1e-6);

vec3 fog_coefficientRayleigh = vec3(5.8 * 1e-6, 1.35 * 1e-5, 3.31 * 1e-5);
vec3 fog_coefficientMie = vec3(0.0000030);
uniform float fogStart;
uniform float fogEnd;

float linear_fog_fade(float vertexDistance, float fogStart, float fogEnd)
{
    if (vertexDistance <= fogStart)
    {
        return 1.0;
    }
    else if (vertexDistance >= fogEnd)
    {
        return 0.0;
    }

    return smoothstep(fogEnd, fogStart, vertexDistance);
}

float gbf(in vec3 eyePlayerPos, in float far, in float z0)
{

    float eyeDist = length(eyePlayerPos);
    float borderFogFactor = smoothstep(far * 0.5, far, eyeDist);
    borderFogFactor *= borderFogFactor * borderFogFactor;
    return (eyeDist > far * 0.5) ? borderFogFactor : 0.0;
}

float waterCaustics(vec3 wPos, vec3 lightSource)
{

    vec2 pos = wPos.xz - (lightSource.xz / lightSource.y * wPos.y);
    float caustic = 0.0;
    mat2 rotationMatrix = mat2(vec2(-0.737367, -0.675493), vec2(0.675493, -0.737367));

    const vec2 wave_size[4] = vec2[](vec2(64.0), vec2(32.0, 16.0), vec2(16.0, 32.0), vec2(48.0));

    for (int i = 0; i < 4; i++)
    {

        float small_wave = texture2D(colortex7, mod(rotationMatrix * pos / wave_size[i], 1.0)).x;

        caustic += max(1.0 - sin(1.0 - pow(0.5 + sin(small_wave * 3.0) * 0.5, 25.0)), 0);
    }
    return (caustic / 4) * 0.5;
}

float fogAmount1 = mix(0.1, 5, normWT) + rainStrength;

#include "/lib/fog/fognoloop.glsl"
